package ru.ulstu.aspirant.service;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import ru.ulstu.admin.model.AspirantForm;
import ru.ulstu.aspirant.model.Aspirant;
import ru.ulstu.aspirant.repository.AspirantRepository;
import ru.ulstu.indicator.model.Indicator;
import ru.ulstu.indicator.service.IndicatorService;
import ru.ulstu.model.User;
import ru.ulstu.user.UserService;

import java.util.List;

@Service
public class AspirantService {
    private final AspirantRepository aspirantRepository;
    private final IndicatorService indicatorService;
    private final UserService userService;

    public AspirantService(AspirantRepository aspirantRepository,
                           IndicatorService indicatorService,
                           UserService userService) {
        this.aspirantRepository = aspirantRepository;
        this.indicatorService = indicatorService;
        this.userService = userService;
    }

    public List<Aspirant> getAspirants() {
        return aspirantRepository.findAll();
    }

    public Aspirant getAspirantById(Integer id) {
        return aspirantRepository.findById(id).orElseThrow(() -> new RuntimeException("Aspirant not found by id"));
    }

    public void save(Aspirant aspirant) {
        aspirantRepository.save(aspirant);
    }

    public void deleteAspirant(AspirantForm aspirantForm) {
        aspirantRepository.deleteById(aspirantForm.getId());
    }

    public Aspirant getAspirantByUser(User user) {
        return aspirantRepository.findByUser(user);
    }

    public List<Indicator> getCurrentAspirantIndicators() {
        Aspirant aspirant = getAspirantByUser(userService.getCurrentUser());
        return indicatorService.getIndicatorsByCourse(aspirant.getCourse());
    }

    public Page<Indicator> getIndicatorsByCourse(Pageable pageable) {
        return indicatorService.getIndicatorsByCourse(pageable);
    }
}
