package ru.ulstu.report.service;

import org.springframework.stereotype.Service;
import ru.ulstu.file.model.FileData;
import ru.ulstu.file.service.FileService;
import ru.ulstu.indicator.service.IndicatorService;
import ru.ulstu.report.model.ReportStatus;
import ru.ulstu.report.model.ReportValue;
import ru.ulstu.report.model.dto.ReportValueDto;
import ru.ulstu.report.repository.ReportValueRepository;
import ru.ulstu.user.UserService;

import java.io.IOException;
import java.util.List;

@Service
public class ReportValueService {
    private final ReportValueRepository reportValueRepository;
    private final IndicatorService indicatorService;
    private final FileService fileService;
    private final ReportService reportService;
    private final UserService userService;

    public ReportValueService(ReportValueRepository reportValueRepository,
                              IndicatorService indicatorService,
                              FileService fileService,
                              ReportService reportService, UserService userService) {
        this.reportValueRepository = reportValueRepository;
        this.indicatorService = indicatorService;
        this.fileService = fileService;
        this.reportService = reportService;
        this.userService = userService;
    }

    public ReportValue saveReportValue(ReportValueDto reportValueDto) throws IOException {
        ReportValue reportValue;
        if (reportValueDto.getId() == null || reportValueDto.getId() == 0) {
            reportValue = new ReportValue();
        } else {
            reportValue = getReportValueById(reportValueDto.getId());
        }
        reportValue.setIndicatorValue(reportValueDto.getIndicatorValue());
        reportValue.setIndicator(indicatorService.getIndicatorById(reportValueDto.getIndicator().getId()));
        List<FileData> files = fileService.saveOrCreate(reportValueDto.getFiles().stream()
                .filter(f -> !f.isDeleted())
                .toList());
        reportValue.getFiles().removeAll(files);
        reportValue.setFiles(files);
        reportValue.setComment(reportValueDto.getComment());
        reportValue = reportValueRepository.save(reportValue);
        reportService.addReportValue(reportValueDto.getReportId(), reportValue);

        return reportValue;
    }

    public ReportValue getReportValueById(Integer reportValueId) {
        return reportValueRepository
                .findById(reportValueId)
                .orElseThrow(() -> new RuntimeException("Report value not found by id"));
    }

    public void deleteReportValue(Integer id) {
        reportValueRepository.deleteById(id);
    }

    public ReportValueDto getByIndicatorId(Integer reportId, Integer indicatorId) {
        ReportValue reportValue = reportValueRepository.findByReportIdAndIndicatorId(reportId, indicatorId);
        if (reportValue == null) {
            return new ReportValueDto(reportId, indicatorService.getIndicatorById(indicatorId));
        }
        return new ReportValueDto(reportValue, reportId);
    }

    public boolean canEdit(Integer reportId) {
        return reportService.getReportById(reportId).getAspirant().getUser().equals(userService.getCurrentUser());
    }

    public ReportStatus getStatus(Integer reportId) {
        return reportService.getReportById(reportId).getStatus();
    }
}
